require "google_seller_rating/version"

require 'open-uri'
require 'nokogiri'
require 'uri/http'
require 'open_uri_redirections'
require 'public_suffix'
require 'timeout'
require 'openssl'
require 'json'

module GoogleSellerRating

  class Api

    attr_accessor :domain, :tld, :timeout, :format

    def initialize(options={})
      @domain  = options[:domain] ||= "http://www.amazon.com/"
      @tld     = options[:tld] ||= "com"
      @timeout = options[:timeout] ||= 5
      @format  = options[:format] ||= "json"
    end

    def rating
      begin
        domain        = parse_domain(@domain)
        html_response = load_google(parse_domain(@domain))
        html_body     = Nokogiri::HTML(html_response[1])
        rating        = html_body.css("#rating").text
        reviews       = html_body.css("#review-count").text.force_encoding("iso-8859-1").gsub(/[^\d]/, "").strip.to_i
        rating = if html_response[0] == "OK"
          { format: @format, status: "OK", google: @tld, domain: domain, rating: rating, reviews: reviews }
        else
          { format: @format, status: html_response[0], google: @tld, domain: domain }
        end
        case @format
        when "hash"
            rating
          else
            rating.to_json
        end
      rescue Exception => e
        puts "Error: #{e}"
      end
    end

    protected

      def load_google(domain)
        begin
          Timeout.timeout(@timeout) do
            html_body = open("https://www.google.#{@tld}/shopping/seller?q=#{domain}", { allow_redirections: :both, ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE, read_timeout: @timeout })
            [ html_body.status.last, html_body.read ]
          end
        rescue Exception => e
          [ "Error: #{e}", nil ]
        end
      end

      def parse_domain(domain)
        begin
          domain = URI(domain).host
          domain = PublicSuffix.parse(domain)
          domain.domain
        rescue Exception => e
          puts "Error: #{e}"
        end
      end

  end

end
